﻿# Export restore points

# Scripts for exporting restore points from Veeam for specified servers and match users from VMware vSphere

# Copyright (C) 2016 Petr Havlíček <petr.havlicek@vsb.cz>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# Load PowerCLI
Add-PSSnapin "Vmware.VimAutomation.Core"
# Load Veeam Snapin
Add-PSSnapin "VeeamPSSnapin"

$vcenter = "vcs.vsb.cz"
$vcenter_user = "vSphere.local\veeam_reports" # This username must exist in Veeam Credentials Database

$date = Get-Date -format yyyyMMddHHmmss

$file = "C:\Users\Administrator\Desktop\powershell\reports\veeam_rp-$date.json"
$file_public = "C:\Users\Administrator\Desktop\powershell\public\veeam_rp-last.json"

########################
#### VEEAM SECTION #####
########################
# Connect to Veeam Server
Connect-VBRServer

# Hash table for VMs to aggregate restore points
$vm_restore_points = @{ "server" = @{} }

# List for Backup Copy job IDs
$backup_copy_job = @()

# Find all Backup Copy jobs
Get-VBRBackup | % {
    if( ([string]$_.JobType).Contains( "Sync" ) ) {
        $backup_copy_job += $_.ID
    }
}

# Cycle over all availible restore points
Get-VBRRestorePoint | % {
    $vm_name = $_.VmName

    # Test of restore point health
    if( $_.IsConsistent -and !$_.IsCorrupted ) {
        
        # Create hash entry for restore point
        $rp = @{
            "type" = [string]$_.Type; `
            "create" = [string]$_.CreationTime; `
        }

        # Detect location
        $rp_location = "local"

        if( $backup_copy_job -contains $_.BackupId) {
            $rp_location = "remote"
        }

        # Load previous restore points
        if( $vm_restore_points[ "server" ].ContainsKey( $vm_name ) -and $vm_restore_points[ "server" ][ $vm_name ].ContainsKey( $rp_location ) ) {
            $vm_restore_points[ "server" ][ $vm_name ][ $rp_location ] += $rp
        }
        elseif( $vm_restore_points[ "server" ].ContainsKey( $vm_name ) ) {
            $vm_restore_points[ "server" ][ $vm_name ][ $rp_location ] = @( $rp )
        }
        else {
            $rp_hash = @{ $rp_location = @( $rp ) }
            $vm_restore_points[ "server" ].Add( $vm_name, $rp_hash )
        }

    }

}

Disconnect-VBRServer

#########################
#### VMWARE SECTION #####
#########################

# Save user credential in encrypted file
# Run this part for change password
#$secure = Read-Host -Prompt "Enter password" -AsSecureString
#ConvertFrom-SecureString -SecureString $secure -Key (1..16) | Set-Content encrypted.txt

# Load credential from file
$pass = Get-Content C:\Users\Administrator\Desktop\powershell\encrypted.txt | ConvertTo-SecureString -Key (1..16)
$cred = New-Object System.Management.Automation.PSCredential ($vcenter_user, $pass)

# Load permission from vSphere
Connect-VIServer $vcenter -Credential $cred

# Cycle over all VMs
Get-VM | %{
    $vm_name = $_.Name
    $permissions = @()

    # Add admins and owner to list
    $permissions += (Get-TagAssignment -Entity $_ -Category Admin).Tag.Name
    $permissions += (Get-TagAssignment -Entity $_ -Category Owner).Tag.Name

    # Select only unique users (for case where admin and owner is same)
    $permissions = @($permissions | Sort -uniq)


    # Write Array to hash table
    if( $vm_restore_points[ "server" ].ContainsKey( $vm_name ) ) {
        $vm_restore_points[ "server" ][ $vm_name ][ "permission" ] = $permissions
    }
}

Disconnect-VIServer -confirm:$false $vcenter


#########################
#### Process Output #####
#########################

# Make JSON file
ConvertTo-Json -Compress -Depth 999 $vm_restore_points | Out-File -Encoding utf8 $file

# Create file with know name for other services
Remove-Item $file_public
Copy-Item $file $file_public